import re
from pydantic import (
    BaseModel,
    StrictStr,
    StrictBool,
    constr
)
from typing import Optional, Literal, Dict


class SecretMount(BaseModel):
    mount_type: Literal["secret"]="secret"
    id: Optional[StrictStr]
    target: Optional[StrictStr]
    required: Optional[StrictBool]
    mode: Optional[constr(max_length=4, pattern=r'^[0-9]*$')]
    user_id: Optional[StrictStr]
    group_id: Optional[StrictStr]

    def to_string(self) -> str:
        mount_string = f"--mount=type={self.mount_type}"

        if self.id:
            mount_string = f'{mount_string},id={self.id}'

        if self.target:
            mount_string = f'{mount_string},target={self.target}'

        if self.required is not None:
            required = "true" if self.required else False
            mount_string = f'{mount_string},required={required}'

        if self.mode:
            mount_string = f'{mount_string},mode={self.mode}'

        if self.user_id:
            mount_string = f'{mount_string},uid={self.user_id}'

        if self.group_id:
            mount_string = f'{mount_string},gid={self.group_id}'

        return mount_string
    
    @classmethod
    def parse(
        cls,
        line: str
    ):
        options: Dict[str, str | bool] = {
            'mount_type': 'secret'
        }

        tokens = line.split(',') 

        for token in tokens:

            if mount_id := re.search(
                r'id=(.*)',
                token
            ):
                options['id'] = re.sub(
                    r'id=',
                    '',
                    mount_id.group(0)
                )

            elif target := re.search(
                r'target=(.*)',
                token
            ):
                options['target'] = re.sub(
                    r'target=',
                    '',
                    target.group(0)
                )

            elif required := re.search(
                r'required=(.*)',
                token
            ):
                required_value = re.sub(
                    r'required=',
                    '',
                    required.group(0)
                )
                options['required'] = True if required_value == 'true' else None
            
            elif mode := re.search(
                r'--mode=[0-7]{4}|[0-7]{3}',
                token
            ):

                options['mode'] = re.sub(
                    r'[0-7]{4}|[0-7]{3}',
                    '',
                    mode.group(0)
                )
            
            elif uid := re.search(
                r'--uid=[0-7]{4}|[0-7]{3}',
                token
            ):
                options['user_id'] = re.sub(
                    r'[0-7]{1,4}',
                    '',
                    uid.group(0)
                )
            
            elif gid := re.search(
                r'--uid=[0-7]{4}|[0-7]{3}',
                token
            ):
                options['group_id'] = re.sub(
                    r'[0-7]{1,4}',
                    '',
                    gid.group(0)
                )

        return SecretMount(**options)


import re
from pydantic import (
    BaseModel,
    StrictStr,
    StrictBool
)

from typing import Optional, Literal, Dict


class BindMount(BaseModel):
    mount_type: Literal["bind"]="bind"
    target: StrictStr
    source: Optional[StrictStr]
    from_source: Optional[StrictStr]
    readwrite: Optional[StrictBool]

    def to_string(self) -> str:
        mount_string = f'--mount=type={self.mount_type},target={self.target}'

        if self.source:
            mount_string = f'{mount_string},source={self.source}'

        if self.from_source:
            mount_string = f'{mount_string},from={self.from_source}'

        if self.readwrite:
            readwrite_string = 'readwrite'
            mount_string = f'{mount_string},{readwrite_string}'

        return mount_string
    
    @classmethod
    def parse(
        cls,
        line: str
    ):
        lines = line.split(' ')
        options: Dict[str, str | bool] = {
            'mount_type': 'bind'
        }

        if 'readwrite' in lines:
            options['readwrite'] = True
            lines.remove('readwrite')

        elif 'rw' in lines:
            options['readwrite'] = True
            lines.remove('rw')

        tokens = line.split(',') 

        for token in tokens:

            if target := re.search(
                r'target=(.*)',
                token
            ):
                options['target'] = re.sub(
                    r'target=',
                    '',
                    target.group(0)
                )

            elif source := re.search(
                r'source=(.*)',
                token
            ):
                options['source'] = re.sub(
                    r'source=',
                    '',
                    source.group(0)
                )

            elif mount_from := re.search(
                r'from=(.*)',
                token
            ):
                options['from_source'] = re.sub(
                    r'from=',
                    '',
                    mount_from.group(0)
                )

            elif readwrite := re.search(
                r'readwrite=(.*)',
                token
            ):
                readwrite_value = re.sub(
                    r'readwrite=',
                    '',
                    readwrite.group(0)
                )
                options['readwrite'] = True if readwrite_value == 'true' else None

        return BindMount(**options)


import re
from pydantic import (
    BaseModel,
    StrictStr
)
from typing import Optional, Literal, Dict


class Stage(BaseModel):
    layer_type: Literal["stage"]="stage"
    base: StrictStr
    tag: StrictStr
    alias: Optional[StrictStr]=None

    def to_string(self) -> str:
        
        if self.alias:
            return f'FROM {self.base}:{self.tag} as {self.alias}'
        
        return f'FROM {self.base}:{self.tag}'
    
    @classmethod
    def parse(
        cls,
        line: str
    ):
        
        line = re.sub('FROM', '', line).strip()
        options: Dict[str, str] = {}

        if alias := re.search(
            r'as(.*)',
            line
        ):
            options['alias'] = alias.group(0)
            line = re.sub(
                r'as(.*)',
                '',
                line
            )

        base, tag = line.split(':')

        return Stage(
            base=base,
            tag=tag,
            **options
        )


import re
from pydantic import (
    BaseModel,
    StrictStr,
    StrictInt
)
from typing import Optional, Literal, Dict


class TMPFSMount(BaseModel):
    mount_type: Literal["tmpfs"]="tmpfs"
    target: StrictStr
    size: Optional[StrictInt]

    def to_string(self) -> str:
        mount_string = f'--mount=type={self.mount_type},target={self.target}'

        if self.size:
            mount_string = f'{mount_string},size={self.size}'

        return mount_string
    
    @classmethod
    def parse(
        cls,
        line: str
    ):
        options: Dict[str, str | bool] = {
            'mount_type': 'tmpfs'
        }

        tokens = line.split(',') 

        for token in tokens:

            if target := re.search(
                r'target=(.*)',
                token
            ):
                options['target'] = re.sub(
                    r'target=',
                    '',
                    target.group(0)
                )

            elif size := re.search(
                r'size=[0-9]{*}',
                token
            ):
                options['size'] = cls._match_size(
                    size.group(0)
                )

        return TMPFSMount(**options)
    
    @classmethod
    def _match_size(
        cls,
        token: str
    ):
        if size := re.search(
            r'[0-9]{*}',
            token
        ):
            return int(size.group(0))
        


import re
from pydantic import (
    BaseModel,
    StrictStr,
    StrictInt,
    StrictBool,
    StrictFloat
)
from typing import Union, Literal


class Env(BaseModel):
    layer_type: Literal["env"]="env"
    key: StrictStr
    value: Union[StrictStr, StrictInt, StrictBool, StrictFloat]

    def to_string(self) -> str:

        value = self.value
        if isinstance(value, str):
            value = f'"{value}"'

        return f'ENV {self.key}={value}'
    
    @classmethod
    def parse(
        cls,
        line: str
    ):
        
        line = re.sub('ENV', '', line)
        token = line.strip()

        key, value = token.split('=')

        return Env(
            key=key,
            value=value
        )


import re
from pydantic import (
    BaseModel,
    StrictStr,
    StrictBool,
    constr
)

from typing import Optional, Literal, Dict


class CacheMount(BaseModel):
    mount_type: Literal["cache"]="cache"
    id: StrictStr
    target: StrictStr
    source: Optional[StrictStr]
    from_source: Optional[StrictStr]
    readonly: Optional[StrictBool]
    sharing: Literal["shared", "private", "locked"]
    mode: Optional[constr(max_length=4, pattern=r'^[0-7]*$')]
    user_id: Optional[StrictStr]
    group_id: Optional[StrictStr]

    def to_string(self) -> str:
        mount_string = f'--mount=type={self.mount_type},target={self.target}'

        if self.id:
            mount_string = f'{mount_string},id={self.id}'

        if self.source:
            mount_string = f'{mount_string},source={self.source}'

        if self.from_source:
            mount_string = f'{mount_string},from={self.from_source}'

        if self.readonly is not None:
            readonly = 'true' if self.readonly else 'false'
            mount_string = f'{mount_string},readonly={readonly}'

        if self.sharing:
            mount_string = f'{mount_string},sharing={self.sharing}'

        if self.mode:
            mount_string = f'{mount_string},mode={self.mode}'

        if self.user_id:
            mount_string = f'{mount_string},uid={self.user_id}'

        if self.group_id:
            mount_string = f'{mount_string},gid={self.group_id}'

        return mount_string
    
    @classmethod
    def parse(
        cls,
        line: str
    ):
        
        lines = line.split(' ')
        options: Dict[str, str | bool] = {
            'mount_type': 'cache'
        }

        if 'readonly' in lines:
            options['readonly'] = True
            lines.remove('readonly')

        elif 'ro' in lines:
            options['readonly'] = True
            lines.remove('ro')

        tokens = line.split(',') 

        for token in tokens:

            if mount_id := re.search(
                r'id=(.*)',
                token
            ):
                options['id'] = re.sub(
                    r'id=',
                    '',
                    mount_id.group(0)
                )

            elif target := re.search(
                r'target=(.*)',
                token
            ):
                options['target'] = re.sub(
                    r'target=',
                    '',
                    target.group(0)
                )

            elif source := re.search(
                r'source=(.*)',
                token
            ):
                options['source'] = re.sub(
                    r'source=',
                    '',
                    source.group(0)
                )

            elif mount_from := re.search(
                r'from=(.*)',
                token
            ):
                options['from_source'] = re.sub(
                    r'from=',
                    '',
                    mount_from.group(0)
                )

            elif readonly := re.search(
                r'readonly=(.*)',
                token
            ):
                readonly_value = re.sub(
                    r'readonly=',
                    '',
                    readonly.group(0)
                )
                options['readonly'] = True if readonly_value == 'true' else None
            
            elif sharing := re.search(
                r'sharing=(shared|private|locked)',
                token
            ):
                options['sharing'] = re.sub(
                    r'shared|private|locked',
                    '',
                    sharing.group(0)
                )
            
            elif mode := re.search(
                r'--mode=[0-7]{4}|[0-7]{3}',
                token
            ):

                options['mode'] = re.sub(
                    r'[0-7]{4}|[0-7]{3}',
                    '',
                    mode.group(0)
                )
            
            elif uid := re.search(
                r'--uid=[0-7]{4}|[0-7]{3}',
                token
            ):
                options['user_id'] = re.sub(
                    r'[0-7]{1,4}',
                    '',
                    uid.group(0)
                )
            
            elif gid := re.search(
                r'--uid=[0-7]{4}|[0-7]{3}',
                token
            ):
                options['group_id'] = re.sub(
                    r'[0-7]{1,4}',
                    '',
                    gid.group(0)
                )

        return CacheMount(**options)
    